@@ -238,14 +238,12 @@ cxxSurface::read_raw(CParser & parser, bool check)
 	std::istream::pos_type ptr;
 	std::istream::pos_type next_char;
 	std::string token;
-	int opt_save;
 	bool useLastLine(false);
 
 	// Read surface number and description
 	this->read_number_description(parser);
 	this->Set_new_def(false);
 
-	opt_save = CParser::OPT_ERROR;
 	bool only_counter_ions_defined(false);
 	bool thickness_defined(false);
 	bool type_defined(false);
@@ -516,7 +514,6 @@ cxxSurface::read_raw(CParser & parser, bool check)
 					("Expected element name and molality for Surface totals.",
 					 PHRQ_io::OT_CONTINUE);
 			}
-			opt_save = 17;
 			break;
 		}
 		if (opt == CParser::OPT_EOF || opt == CParser::OPT_KEYWORD)
