@@ -329,8 +329,13 @@ print_diffuse_layer(cxxSurfaceCharge *charge_ptr)
 	}
 	else
 	{
+		LDBLE exp_g =  charge_ptr->Get_g_map()[1].Get_g() * mass_water_aq_x / mass_water_surface + 1;
+		LDBLE psi_DL = -log(exp_g) * R_KJ_DEG_MOL * tk_x / F_KJ_V_EQ;
 		output_msg(sformatf(
-				   "\n\tTotal moles in diffuse layer (excluding water), Donnan calculation \n\n"));
+				   "\n\tTotal moles in diffuse layer (excluding water), Donnan calculation."));
+		output_msg(sformatf(
+				   "\n\tDonnan Layer potential, psi_DL = %10.3e V.\n\tBoltzmann factor, exp(-psi_DL * F / RT) = %9.3e (= c_DL / c_free if z is +1).\n\n",
+					 psi_DL, exp_g));
 	}
 	output_msg(sformatf("\tElement       \t     Moles\n"));
 	for (j = 0; j < count_elts; j++)
@@ -1577,10 +1582,18 @@ print_surface(void)
  */
 
 	s_h2o->lm = s_h2o->la;
+	if (use.Get_surface_ptr()->Get_type() == cxxSurface::DDL)
+	{
+		output_msg(sformatf("%-14s\n", "Diffuse Double Layer Surface-Complexation Model\n"));
+	}
+	else if (use.Get_surface_ptr()->Get_type() == cxxSurface::CCM)
+	{
+		output_msg(sformatf("%-14s\n", "Constant Capacitance Surface-Complexation Model\n"));
+	}
 	for (int j = 0; j < count_unknowns; j++)
 	{
 		/*if (use.Get_surface_ptr()->edl == TRUE) { */
-		if (use.Get_surface_ptr()->Get_type() == cxxSurface::DDL)
+		if (use.Get_surface_ptr()->Get_type() == cxxSurface::DDL || use.Get_surface_ptr()->Get_type() == cxxSurface::CCM)
 		{
 			if (x[j]->type != SURFACE_CB)
 				continue;
@@ -1608,7 +1621,7 @@ print_surface(void)
 					   (double) x[j]->f));
 		}
 		/*if (use.Get_surface_ptr()->edl == TRUE && diffuse_layer_x == FALSE) { */
-		if (use.Get_surface_ptr()->Get_type() == cxxSurface::DDL && dl_type_x == cxxSurface::NO_DL)
+		if ((use.Get_surface_ptr()->Get_type() == cxxSurface::DDL || use.Get_surface_ptr()->Get_type() == cxxSurface::CCM) && dl_type_x == cxxSurface::NO_DL)
 		{
 			charge = x[j]->f;
 		}
@@ -1641,6 +1654,11 @@ print_surface(void)
 				output_msg(sformatf("\tundefined  sigma, C/m\n"));
 #endif
 			}
+			if (use.Get_surface_ptr()->Get_type() == cxxSurface::CCM)
+			{			
+				output_msg(sformatf("\t%11.3e  capacitance, F/m^2\n",
+					   (double) (charge_ptr->Get_capacitance0())));
+			}
 			output_msg(sformatf("\t%11.3e  psi, V\n",
 					   (double) (x[j]->master[0]->s->la * 2 * R_KJ_DEG_MOL *
 								 tk_x * LOG_10 / F_KJ_V_EQ)));
@@ -2225,6 +2243,23 @@ print_totals(void)
 			   (double) calc_solution_volume()));
 	}
 /* VP: Density End */
+#ifdef NPP
+	if (print_viscosity)
+	{
+		output_msg(sformatf("%45s%9.5f", "Viscosity (mPa s)  = ",
+			   (double) viscos));
+		if (tc_x > 200 && !pure_water) 
+		{
+			output_msg(sformatf("%18s\n", 
+#ifdef NO_UTF8_ENCODING
+				   " (solute contributions limited to 200 oC)"));
+#else
+				   " (solute contributions limited to 200 C)"));
+#endif
+		}
+		else output_msg(sformatf("\n"));
+	}
+#endif
 	output_msg(sformatf("%45s%7.3f\n", "Activity of water  = ",
 			   exp(s_h2o->la * LOG_10)));
 	output_msg(sformatf("%45s%11.3e\n", "Ionic strength  = ",
